<?php

global $langs, $conffile, $db, $dolibarr_main_db_pass, $dolibarr_main_db_encrypted_pass;

// Check if install lock file is present
$lockfile = DOL_DATA_ROOT . '/install.lock';
$lockfileexists = ! (! empty($lockfile) && ! file_exists($lockfile) && is_dir(DOL_DOCUMENT_ROOT . '/install'));

// Conf files must be in read only mode
$conffileisreadonly = ! is_writable(DOL_DOCUMENT_ROOT . '/' . $conffile);

// Check database password
$databasepwdisempty = empty($dolibarr_main_db_pass) && empty($dolibarr_main_db_encrypted_pass);

// Check users passwords
// @see https://en.wikipedia.org/wiki/List_of_the_most_common_passwords
$vulnerable_users = array();
$vulnerable_passwords = array(
	'admin',
	'master',
	'user',
	'demo',
	'login',
	'azerty',
	'qwerty',
	'qazwsx',
	'zxcvbnm',
	'password',
	'passw0rd',
	'password1',
	'abc123',
	'1234',
	'12345',
	'123456',
	'1234567',
	'12345678',
	'123456789',
	'1234567890',
	'987654321',
	'654321',
	'696969',
	'123123',
	'123321',
	'12341234',
	'000000',
	'111111',
	'555555',
	'666666',
	'7777777',
	'20182018',
	'2018'
);

// Dolibarr < 8.0.0
if (! function_exists('dol_verifyHash'))//compare_version(DOL_VERSION, '<', '8.0.0'))
{
	$hashes = array();

	foreach ($vulnerable_passwords as $password) {
		$hashes[] = dol_hash($password);
	}

	$sql = "SELECT rowid, login, firstname, lastname FROM " . MAIN_DB_PREFIX . "user WHERE pass_crypted IN('" . join("', '", $hashes) . "')";
	$resql = $db->query($sql);
	if ($resql)
	{
		$count = $db->num_rows($resql);
		if ($count > 0)
		{
			$i = 0;

			while ($i < $count)
			{
				$obj = $db->fetch_object($resql);
				$vulnerable_users[] = array(
					'rowid' => $obj->rowid,
					'login' => $obj->login,
					'firstname' => $obj->firstname,
					'lastname' => $obj->lastname,
				);

				$i++;
			}
		}
	}
}
// Dolibarr >= 8.0.0
else
{
	$sql = "SELECT rowid, login, firstname, lastname, pass_crypted FROM " . MAIN_DB_PREFIX . "user";
	$resql = $db->query($sql);
	if ($resql)
	{
		$count = $db->num_rows($resql);
		if ($count > 0)
		{
			$i = 0;

			while ($i < $count)
			{
				$obj = $db->fetch_object($resql);

				foreach ($vulnerable_passwords as $password)
				{
					if (dol_verifyHash($password, $obj->pass_crypted)) {
						$vulnerable_users[] = array(
							'rowid' => $obj->rowid,
							'login' => $obj->login,
							'firstname' => $obj->firstname,
							'lastname' => $obj->lastname,
						);
					}
				}

				$i++;
			}
		}
	}
}

// Translate strings
$addinstalllock = $langs->trans('AddInstallLock');
$removeinstalllock = $langs->trans('RemoveInstallLock');
$setconftoreadonly = $langs->trans('SetConfToReadOnly');
$setconftowritable = $langs->trans('SetConfToWritable');
$changepwdstr = $langs->trans('ChangePassword');

// Set button icons
$usefontawesomeicons = compare_version(DOL_VERSION, '>=' ,'7.0.0');
$fixicon = $usefontawesomeicons ? '<i class="fa fa-wrench fa-fw paddingright"></i>' : '';
$removeicon = $usefontawesomeicons ? '<i class="fa fa-remove fa-fw paddingright"></i>' : '';
$editicon = $usefontawesomeicons ? '<i class="fa fa-edit fa-fw paddingright"></i>' : '';

?>

<table class="noborder allwidth">
	<tr class="liste_titre">
		<td width="70%"><?php echo $langs->trans('Issue'); ?></td>
		<td align="center"><?php echo $langs->trans('Severity'); ?></td>
		<td align="center"></td>
	</tr>
	<tr>
		<td><?php
			echo $langs->trans('LockFileDoesNotExists', DOL_DATA_ROOT);
		?></td>
		<td align="center"><?php
			if ($lockfileexists) {
				echo img_picto($langs->trans('Solved'), 'solved.png@securemydolibarr', 'class="smd-icon"');
			} else {
				echo img_picto($langs->trans('Critical'), 'critical.png@securemydolibarr', 'class="smd-icon"');
			}
		?></td>
		<td align="center">
			<?php if (! $lockfileexists) { ?>
				<a href="<?php echo $_SERVER["PHP_SELF"].'?action=add_install_lock'; ?>" class="smd-button green" title="<?php echo $addinstalllock; ?>">
					<?php echo $fixicon.$addinstalllock; ?>
				</a>
			<?php } else if (js_enabled()) { ?>
				<span id="remove-install-lock" class="smd-button red" title="<?php echo $removeinstalllock; ?>">
					<?php echo $removeicon.$removeinstalllock; ?>
				</span>
			<?php } else { ?>
				<a href="<?php echo $_SERVER["PHP_SELF"].'?action=remove_install_lock'; ?>" class="smd-button red" title="<?php echo $removeinstalllock; ?>">
					<?php echo $removeicon.$removeinstalllock; ?>
				</a>
			<?php } ?>
		</td>
	</tr>
	<tr>
		<td><?php
			echo $langs->trans('ConfFileMustBeReadOnly');
		?></td>
		<td align="center"><?php
			if ($conffileisreadonly) {
				echo img_picto($langs->trans('Solved'), 'solved.png@securemydolibarr', 'class="smd-icon"');
			} else {
				echo img_picto($langs->trans('Critical'), 'critical.png@securemydolibarr', 'class="smd-icon"');
			}
		?></td>
		<td align="center">
			<?php if (! $conffileisreadonly) { ?>
				<a href="<?php echo $_SERVER["PHP_SELF"].'?action=chmod_conf_to_read_only'; ?>" class="smd-button green" title="<?php echo $setconftoreadonly; ?>">
					<?php echo $fixicon.$setconftoreadonly; ?>
				</a>
			<?php } else if (js_enabled()) { ?>
				<span id="chmod-conf-to-writable" class="smd-button red" title="<?php echo $setconftowritable; ?>">
					<?php echo $removeicon.$setconftowritable; ?>
				</span>
			<?php } else { ?>
				<a href="<?php echo $_SERVER["PHP_SELF"].'?action=chmod_conf_to_writable'; ?>" class="smd-button red" title="<?php echo $setconftowritable; ?>">
					<?php echo $removeicon.$setconftowritable; ?>
				</a>
			<?php } ?>
		</td>
	</tr>
	<?php if ($databasepwdisempty) { ?>
		<tr>
			<td><?php
				echo $langs->trans('DatabasePassIsEmpty');
			?></td>
			<td align="center"><?php
				echo img_picto($langs->trans('Critical'), 'critical.png@securemydolibarr', 'class="smd-icon"');
			?></td>
			<td align="center">
				&nbsp;
			</td>
		</tr>
	<?php } ?>
	<?php foreach ($vulnerable_users as $user) { ?>
		<tr>
			<td><?php
				$username = empty($user['lastname']) ? $user['login'] : $user['lastname'].(empty($user['firstname']) ? '' : ' '.$user['firstname']);
				echo $langs->trans('UserPasswordMustBeChanged', $username);
			?></td>
			<td align="center"><?php
				echo img_picto($langs->trans('Potential'), 'potential.png@securemydolibarr', 'class="smd-icon"');
			?></td>
			<td align="center">
				<a href="<?php echo DOL_URL_ROOT . '/user/card.php?id='.$user['rowid'].'&action=edit'; ?>" class="smd-button blue" title="<?php echo $changepwdstr; ?>">
					<?php echo $editicon.$changepwdstr; ?>
				</a>
			</td>
		</tr>
	<?php } ?>
</table>

<br>

<!-- Legend -->

<div>
	<?php echo img_picto($langs->trans('Critical'), 'critical.png@securemydolibarr', 'class="smd-icon small"'); ?>
	<span class="smd-legend"><?php echo $langs->trans('CriticalDescription'); ?></span>
</div>

<div>
	<?php echo img_picto($langs->trans('Potential'), 'potential.png@securemydolibarr', 'class="smd-icon small"'); ?>
	<span class="smd-legend"><?php echo $langs->trans('PotentialDescription'); ?></span>
</div>

<div>
	<?php echo img_picto($langs->trans('Solved'), 'solved.png@securemydolibarr', 'class="smd-icon small"'); ?>
	<span class="smd-legend"><?php echo $langs->trans('SolvedDescription'); ?></span>
</div>
